
.PROGRAM OR_init.pc(.init_cfgID)
;FUNCTION: Initializing EIP communication 
;@param .init_cfgID Tool configuration ID, see the possible configurations in the table below
;@param OR_init_fbCh Fieldbus channel: 1-4
;
;cfgID		Primary 	   Secondary
;-------------------------------------------------------------------
;101	-	 RG2FT		    -
;102	-	 RGx		    -
;103	-	 VG10/VGC10		-
;104	-	 Gecko		    -
;105	HEX	 -		        -
;106	HEX	 RGx		    -
;107	HEX	 VG10/VGC10		-
;108	HEX	 Gecko		    -
;109	-	 RGx		    VG10/VGC10
;110	-	 RGx		    Gecko
;111	-	 RGx		    RGx
;112	-	 VG10/VGC10		RGx
;113	-	 VG10/VGC10		Gecko
;114	-	 VG10/VGC10		VG10/VGC10

	;Select Fieldbus (Ethernet/IP) bit offset
	OR_R_bitOffset = 32
	OR_W_bitOffset = 32

	;RESET ALL ROBOT OUTPUTS TO GRIPPER 
	FOR i = (1+OR_W_bitOffset) TO (OR_W_bitOffset+512)
		SIG -(i)
	END

	;Select the correct bit offsert according to tool configuration
	CASE .init_cfgID OF
		VALUE 101, 102, 103, 104, 105
			primR_bitOffset = 0 + 1000 + OR_R_bitOffset
			primW_bitOffset = 0 + OR_W_bitOffset
			secR_bitOffset = 0 + 1000 + OR_R_bitOffset
			secW_bitOffset = 0 + OR_W_bitOffset
	
		VALUE 106, 107, 108
			primR_bitOffset = 192 + 1000 + OR_R_bitOffset
			primW_bitOffset = 128 + OR_W_bitOffset
			secR_bitOffset = 192 + 1000 + OR_R_bitOffset
			secW_bitOffset = 128  + OR_W_bitOffset

		VALUE 109, 110, 113
			primR_bitOffset = 0 + 1000 + OR_R_bitOffset
			primW_bitOffset = 0 + OR_W_bitOffset
			secR_bitOffset = 384 + 1000 + OR_R_bitOffset
			secW_bitOffset = 384 + OR_W_bitOffset

		VALUE 111, 114
			primR_bitOffset = 0 + 1000 + OR_R_bitOffset
			primW_bitOffset = 0 + OR_W_bitOffset
			secR_bitOffset = 128 + 1000 + OR_R_bitOffset
			secW_bitOffset = 128 + OR_W_bitOffset

		VALUE 112
			primR_bitOffset = 256 + 1000 + OR_R_bitOffset
			primW_bitOffset = 256 + OR_W_bitOffset
			secR_bitOffset = 128 + 1000 + OR_R_bitOffset
			secW_bitOffset = 128 + OR_W_bitOffset
	END
.END


.PROGRAM OR_Gecko_padOut.pc(.inst, .wait)
;FUNCTION: Move out gecko pads
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@param .wait 0 = don;t wait for pad movement, 1 = wait until pad reach the designated position

	.padOut_oW = 0
	.padOut_oR = 0
	
	IF .inst == 1 THEN
		.padOut_oW = primW_bitOffset
		.padOut_oR = primR_bitOffset
	ELSE
		.padOut_oW = secW_bitOffset
		.padOut_oR = secR_bitOffset
	END

	BITS (1 + .padOut_oW), 16 = 1 ;Reset pad in bit, Set pad out bit
	TWAIT  0.1

	BITS (1 + .padOut_oW), 16 = 0 ;Reset pad out bit after some time

	IF .wait THEN
		SWAIT (19 + .padOut_oR)
	END
.END

.PROGRAM OR_Gecko_padIn.pc(.inst, .wait)
;FUNCTION: Move in gecko pads
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@param .wait 0 = don;t wait for pad movement, 1 = wait until pad reach the designated position

	.padIn_oW = 0
	.padIn_oR = 0
	
	IF .inst == 1 THEN
		.padIn_oW = primW_bitOffset
		.padIn_oR = primR_bitOffset
	ELSE
		.padIn_oW = secW_bitOffset
		.padIn_oR = secR_bitOffset
	END

	BITS (1 + .padIn_oW), 16 = 2 ;Reset pad out bit, Set pad in  bit
	TWAIT  0.1
	BITS (1 + .padIn_oW), 16 = 2 ;Reset pad in bit after some time

	IF .wait THEN
		SWAIT -(19 + .padIn_oR)
	END

.END

.PROGRAM OR_Gecko_getF.pc(.inst, .getF_r) 
;FUNCTION: Get preload force
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@ret preload force in N
	.getF_a = 0 ;address
	.getF_r = 0 ;return value

	;Set bit address
	.getF_a = 49

	;Set offset according primary (single) or secondary
	IF .inst == 1 THEN
		.getF_a = .getF_a + primR_bitOffset
	ELSE
		.getF_a = .getF_a + secR_bitOffset
	END
	
	;Get data
	.getF_r = BITS(.getF_a, 16)
	.getF_r = .getF_r / 100 ;get N from 1/100N
.END


.PROGRAM OR_Gecko_getUS.pc(.inst, .getUS_r)
;FUNCTION: Get ultrasonic sensor data
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@ret Part distance in mm

	.getUS_a = 0 ;address
	.getUS_r = 0 ;return value

	;Set bit address
	.getUS_a = 65

	;Set offset according primary (single) or secondary
	IF .inst == 1 THEN
		.getUS_a = .getUS_a + primR_bitOffset
	ELSE
		.getUS_a = .getUS_a + secR_bitOffset
	END
	
	;Get data
	.getUS_r = BITS(.getUS_a, 16)

	.getUS_r = .getUS_r / 100 ;get mm from 1/100mm

.END

.PROGRAM OR_Gecko_isConn.pc(.inst, .isConn_r)
;FUNCTION: Check, that is device connected or not
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@ret 0 = not connected, 1 = device connected

	.isConn_a = 0 ;address
	.isConn_r = 0 ;return value

	;Set bit address
	.isConn_a = 1

	;Set offset according primary (single) or secondary
	IF .inst == 1 THEN
		.isConn_a = .isConn_a + primR_bitOffset
	ELSE
		.isConn_a = .isConn_a + secR_bitOffset
	END
	
	;Get data
	.isConn_r = BITS(.isConn_a, 16)
	
	IF .isConn_r == 48 THEN
		.isConn_r = 1
	ELSE
		.isConn_r = 0
	END

.END

.PROGRAM OR_Gecko_isPart.pc( .inst, .isPart_r)
;FUNCTION: Check if part detected or not
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@ret 0 = part not detected, 1 = part detected

	.isPart_a = 0 ;address
	.isPart_r = 0 ;return value

	;Set bit address
	.isPart_a = 17

	;Set offset according primary (single) or secondary
	IF .inst == 1 THEN
		.isPart_a = .isPart_a + primR_bitOffset
	ELSE
		.isPart_a = .isPart_a + secR_bitOffset
	END
	
	;Get data
	.isPart_r = -SIG(.isPart_a)

.END

.PROGRAM OR_Gecko_padSt.pc(.inst, .padSt_r)
;FUNCTION: Get pad worn state
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@ret 0 = pads are good (no error), 1 = pads are worn

	.padSt_a = 0 ;address
	.padSt_r = 0 ;return value

	;Set bit address
	.padSt_a = 18

	;Set offset according primary (single) or secondary
	IF .inst == 1 THEN
		.padSt_a = .padSt_a + primR_bitOffset
	ELSE
		.padSt_a = .padSt_a + secR_bitOffset
	END
	
	;Get data
	.padSt_r = -SIG(.padSt_a)

.END

.PROGRAM OR_Gecko_padPos.pc(.inst, .padPos_r)
;FUNCTION: Get pad position
;@param .inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
;@ret 1=pad out, 0=pad in

	.padPos_a = 0 ;address
	.padPos_r = 0 ;return value

	;Set bit address
	.padPos_a = 19

	;Set offset according primary (single) or secondary
	IF .inst == 1 THEN
		.padPos_a = .padPos_a + primR_bitOffset
	ELSE
		.padPos_a = .padPos_a + secR_bitOffset
	END
	
	;Get data
	.padPos_r = -SIG(.padPos_a)

.END






















.PROGRAM OR_HEX_get.pc(.$get_f, .get_r)
;FUNCTION: Get HEX values
;@param .get_f: reqested force/torque instance, valid inputs: Fx, Fy, Fz, Tx, Ty, Tz
;force values are in 1/10N, torque values are in 1/100Nm
;eg.: Fz = sensor force value on z axis, Tx = sensor torque value on x axis

	.get_r = 0 ;return value
	.get_a = 0 ;address
	
	SCASE .$get_f OF
	SVALUE "Fx"
		.get_a = 65
	SVALUE "Fy"
		.get_a = 81
	SVALUE "Fz"
		.get_a = 97
	SVALUE "Tx"
		.get_a = 113
	SVALUE "Ty"
		.get_a = 129
	SVALUE "Tz"
		.get_a = 145
	ANY:
		.get_r = 0
	END

	.get_a = .get_a + OR_R_bitOffset + 1000
	
	IF .get_a > 0 THEN
		.get_r = BITS(.get_a, 16)
		IF .get_r >= 32768 THEN
			.get_r = .get_r - 65536
		END 

		.get_r = .get_r/10
		SCASE .$get_f OF
		SVALUE "Tx", "Ty", "Tz"
			.get_r = .get_r/10
		END
	END

.END

.PROGRAM OR_HEX_zero.pc()
;FUNCTION: Zeroing the HEX sensor values (the actual force/torque will be 0)

	BITS (1 + OR_W_bitOffset), 16 = 1
	TWAIT  0.1
	BITS (1 + OR_W_bitOffset), 16 = 0

.END

.PROGRAM OR_HEX_unzero.pc()
;FUNCTION: Unzero the HEX sensor

	BITS (1 + OR_W_bitOffset), 16 = 2
	TWAIT  0.1
	BITS (1 + OR_W_bitOffset), 16 = 0

.END

.PROGRAM OR_HEX_isConn.pc(.isConn_r) 
;FUNCTION: Check that is device connected or not
;@ret 1 = connected, 0 = not connected

	.isConn_a  = 0 ;address
	.isConn_r  = 0 ;return value

	;Set bit address
	.isConn_a = 1 + 1000 + OR_R_bitOffset

	;Get data
	.isConn_r = BITS(.isConn_a, 16)

	IF .isConn_r == 64 THEN
		.isConn_r = 1
	ELSE
		.isConn_r = 0
	END
.END

















.PROGRAM OR_RG2FT_move.pc( .move_width , .move_force , .move_wait)
;FUNCTION: Open/Close gripper to specific dictance (mm) with specific force (N)
;@param .move_width open/close distance (mm)
;@param .move_force force (N)
;@param .move_wait shall we wait for command execution? 1=YES, 0=NO

	.move_addr  = 0
	.move_val  = 0

	;Set width
	.move_addr = 1 + OR_W_bitOffset
	.move_val = .move_width*10 ;Width is sent as 1/10mm 
	BITS .move_addr, 16 = .move_val

	;Set force
	.move_addr = 17 + OR_W_bitOffset 
	.move_val = .move_force*10 ;Force is sent as 1/10N
	BITS .move_addr, 16 = .move_val
	;Start move
	BITS (33 + OR_W_bitOffset), 16 = 1 ;Reset motion stop bit, Set start motion bit
	TWAIT 0.1
	BITS (33 + OR_W_bitOffset), 16 = 0 ;Reset start motion bit

	;Wait for gripper motion if needed
	IF .move_wait == 1 THEN
		TWAIT 0.5
		SWAIT -(449 + OR_R_bitOffset + 1000)
	END

.END


.PROGRAM OR_RG2FT_stop.pc()
;FUNCTION: Stop the gripper motion

	BITS (33 + OR_W_bitOffset), 16 = 2 ;Reset motion start bit, Set stop motion bit
	TWAIT 0.1
	BITS (33 + OR_W_bitOffset), 16 = 0 ;Reset stop motion bit

.END


.PROGRAM OR_RG2FT_pOfAct.pc()
;FUNCTION: Set actual value as offset for proximity

	BITS (81 + OR_W_bitOffset), 16 = 0
	BITS (97 + OR_W_bitOffset), 16 = 0
	BITS (113 + OR_W_bitOffset), 16 = 1
	TWAIT 0.1
	BITS (113 + OR_W_bitOffset), 16 = 0

.END


.PROGRAM OR_RG2FT_pOfVaL.pc( .pOffsVaL_L, .pOffsVaL_R )
;FUNCTION: Set custom offset for proximity
;@param .pOffsVaL_L left proximity offset (mm)
;@param .pOffsVaL_R right proximity offset (mm)

	.pOffsVaL_a  = 0 ;address
	.pOffsVaL_v  = 0 ;value

	;Set left offset value
	.pOffsVaL_a = 81 + OR_W_bitOffset
	BITS .pOffsVaL_a, 16 = .pOffsVaL_L

	;Set right offset value
	.pOffsVaL_a = 97 + OR_W_bitOffset
	BITS .pOffsVaL_a, 16 = .pOffsVaL_R

	;Select offset type

	BITS (113 + OR_W_bitOffset), 16 = 2
	TWAIT 0.1
	BITS (113 + OR_W_bitOffset), 16 = 0

.END


.PROGRAM OR_RG2FT_hxZr.pc()
;FUNCTION: Zero the HEX sensors (actual force/torque values will be zero).

	BITS (49 + OR_W_bitOffset), 16 = 1 ;Reset unzero bit, Set zero bit
	TWAIT 0.1
	BITS (49 + OR_W_bitOffset), 16 = 0 ;Reset zero bit

.END

.PROGRAM OR_RG2FT_hxUnzr.pc()
;FUNCTION: Unzero the values of the HEX sensors (don’t use any offset, reset to the original values)

	BITS (49 + OR_W_bitOffset), 16 = 2 ;Reset zero bit, Set unzero bit
	TWAIT 0.1
	BITS (49 + OR_W_bitOffset), 16 = 0 ;Reset zero bit

.END


.PROGRAM OR_RG2FT_gtLPrx.pc(.getLProx_r) 
;FUNCTION: Get left proximity value (mm)

	.getLProx_r  = 0 ;return value
	.getLProx_a  = 0 ;address
	
	.getLProx_a = 369 + OR_R_bitOffset + 1000
	.getLProx_r = BITS(.getLProx_a, 16)

	IF .getLProx_r  >= 32768 THEN
		.getLProx_r  = .getLProx_r - 65536
	END

.END

.PROGRAM OR_RG2FT_gtRPrx.pc(.getRProx_r) 
;FUNCTION: Get right proximity value (mm)

	.getRProx_r  = 0 ;return value
	.getRProx_a  = 0 ;address
	
	.getRProx_a = 401 + OR_R_bitOffset + 1000
	.getRProx_r = BITS(.getRProx_a, 16)

	IF .getRProx_r  >= 32768 THEN
		.getRProx_r  = .getRProx_r - 65536
	END

.END


.PROGRAM OR_RG2FT_getHex.pc(.$getHex_f, .getHex_r) 
;FUNCTION: Get HEX values
;@param .getHex_f: 	reqested force/torque instance, valid inputs: lFx, lFy, lFz, lTx, lTy, lTz, rFx, rFy, rFz, rTx, rTy, rTz
;				force values are in 1/10N, torque values are in 1/100Nm
;				eg.: lFz = left sensor force value on z axis, rTx = right sensor torque value on x axis

	
	.getHex_r  = 0 ;return value
	.getHex_a  = 0 ;address


	SCASE .$getHex_f OF

	SVALUE "lFx"
		.getHex_a = 65 + OR_R_bitOffset + 1000
	SVALUE "lFy"
		.getHex_a = 81 + OR_R_bitOffset + 1000
	SVALUE "lFz"
		.getHex_a = 97 + OR_R_bitOffset + 1000
	SVALUE "lTx"
		.getHex_a = 113 + OR_R_bitOffset + 1000
	SVALUE "lTy"
		.getHex_a = 129 + OR_R_bitOffset + 1000
	SVALUE "lTz"
		.getHex_a = 145 + OR_R_bitOffset + 1000
	SVALUE "rFx"
		.getHex_a = 241 + OR_R_bitOffset + 1000
	SVALUE "rFy"
		.getHex_a = 257 + OR_R_bitOffset + 1000
	SVALUE "rFz"
		.getHex_a = 273 + OR_R_bitOffset + 1000
	SVALUE "rTx"
		.getHex_a = 289 + OR_R_bitOffset + 1000
	SVALUE "rTy"
		.getHex_a = 305 + OR_R_bitOffset + 1000
	SVALUE "rTz"
		.getHex_a = 321 + OR_R_bitOffset + 1000
	ANY:
		.getHex_a = 0
		.getHex_r = 0
	END

	IF .getHex_a > 0 THEN
		.getHex_r = BITS(.getHex_a, 16)
		IF .getHex_r >= 32768 THEN
			.getHex_r = .getHex_r - 65536
		END
		.getHex_r = .getHex_r/10

		SCASE .$getHex_f OF
		SVALUE "lTx", "lTy", "lTz", "rTx", "rTy", "rTz"
			.getHex_r = .getHex_r/10
		END
	END

.END


.PROGRAM OR_RG2FT_getWid.pc(.getWidth_r) 
;FUNCTION: Get gripper actual width (mm)
;@ret Actual width in mm

	.getWidth_r  = 0 ;return value
	.getWidth_a  = 0 ;address
	
	.getWidth_a = 433 + OR_R_bitOffset + 1000
	.getWidth_r = BITS(.getWidth_a, 16)
	IF .getWidth_r >= 32768 THEN
		.getWidth_r = .getWidth_r - 65536
	END

	.getWidth_r = .getWidth_r/10 ;to mm
.END


.PROGRAM OR_RG2FT_isConn.pc(.isConn_r) 
;FUNCTION: Check that is device connected or not
;@ret 1 = connected, 0 = not connected

	.isConn_a  = 0 ;address
	.isConn_r  = 0 ;return value

	;Set bit address
	.isConn_a = 1  + OR_R_bitOffset + 1000

	;Get data
	.isConn_r = BITS(.isConn_a, 16)
	
	IF .isConn_r == 34 THEN
		.isConn_r = 1
	ELSE
		.isConn_r = 0
	END

.END


.PROGRAM OR_RG2FT_isBusy.pc(.isBusy_r) 
;FUNCTION: Get gripper status (busy)
;@ret 1 = busy, 0= idle

	.isBusy_a  = 0 ;address
	.isBusy_r  = 0 ;return value

	;Set bit address
	.isBusy_a = 449 + OR_R_bitOffset + 1000

	;Get bitstatus
	.isBusy_r = -SIG(.isBusy_a)
.END


.PROGRAM OR_RG2FT_isGrip.pc( .isGrip_r) 
;FUNCTION: Check, that is grip detected or not (part gripped)
;@ret 1 = grip detected, 0 = no grip detected

	.isGrip_a  = 0 ;address
	.isGrip_r  = 0 ;return value

	;Set bit address
	.isGrip_a = 450 + OR_R_bitOffset + 1000

	;Get bitstatus
	.isGrip_r = -SIG(.isGrip_a)
.END

















.PROGRAM OR_VG10_grip.pc(.grip_inst , .grip_vacA , .grip_vacB , .grip_wait  )
;FUNCTION: Grip or release with VG10/VGC10
;@param .grip_inst Gripper instance 1 = primary(or single), 2 = secondary
;@param .grip_vacA Vacuum level of channel A in %
;@param .grip_vacB Vacuum level of channel B in %
;@param .grip_wait Wait until the vacuum builds up: 0=don;t wait, 1=wait untilt the vacuum reaches the required level
	
	.grip_addrA  = 0
	.grip_valA  = 0
	.grip_addrB  = 0
	.grip_valB  = 0
	.grip_bitOffs  = 0
	.grip_vA  = 0
	.grip_vB  = 0

	;Set offset according primary (single) or secondary
	IF .grip_inst == 1 THEN
		.grip_bitOffs = primW_bitOffset
	ELSE
		.grip_bitOffs = secW_bitOffset
	END

	;Control byte lowest bit always on (idle) not used 
	BITS (1 + .grip_bitOffs), 16 = 2
	BITS (17 + .grip_bitOffs), 16 = 2

	;Set vacuum if needed
	IF .grip_vacA > 0 THEN
		.grip_addrA = 33 + .grip_bitOffs
		BITS .grip_addrA, 16 = .grip_vacA
	END

	IF .grip_vacB > 0
		.grip_addrB = 49 + .grip_bitOffs
		BITS .grip_addrB, 16 = .grip_vacB
	END

	;Control gripper according operation types
	IF .grip_vacA == 0 THEN
		BITS (1 + .grip_bitOffs), 16 = 3
	END
	IF .grip_vacA > 0 THEN
		BITS (1 + .grip_bitOffs), 16 = 1
	END
	IF .grip_vacB == 0 THEN
		BITS (17 + .grip_bitOffs), 16 = 3
	END
	IF .grip_vacB > 0 THEN
		BITS (17 + .grip_bitOffs), 16 = 1
	END


	;If  we have to wait for the vacuum reach the achived level
	.cntr = 0
	WHILE .grip_wait DO
		.cntr = .cntr + 1
		TWAIT 0.1
		CALL OR_VG10_getVacA.pc(.grip_inst, .grip_vA) 
		CALL OR_VG10_getVacB.pc(.grip_inst, .grip_vB) 

		IF .grip_vacA == 0 AND .grip_vacB == 0 THEN
			IF .grip_vA <= 1 AND .grip_vB <= 1 THEN
				.grip_wait = 0
			END
		ELSE
			IF .grip_vA >= .grip_vacA AND .grip_vB >= .grip_vacB THEN
				.grip_wait = 0
			END
		END

		IF .cntr > 100 THEN ;time out
			.grip_wait = 0
			PRINT "TIMEOUT WAITING FOR VAC PRESSURE"
		END
	END

.END


.PROGRAM OR_VG10_setCur.pc(.setCur_inst , .setCur_mA )
;FUNCTION: Set current limit for VG10/VGC10
;@param .setCur_inst gripper instance 1 = primary(or single), 2 = secondary
;@param .setCur_mA Current limit in mA, valid range: 100-1000

	.setCur_addr  = 0
	.setCur_addr = 65

	;Shift address according primary (single) or secondary
	IF .setCur_inst == 1 THEN
		.setCur_addr = .setCur_addr + primW_bitOffset
	ELSE
		.setCur_addr = .setCur_addr + secW_bitOffset
	END

	;Write register
	BITS .setCur_addr, 16 = .setCur_mA

.END


.PROGRAM OR_VG10_isConn.pc(.isConn_inst, .isConn_ret )
;FUNCTION: Check VG10/VGC10 connection
;@param .isConn_inst gripper instance 1 = primary(or single), 2 = secondary
;@ret VG10/VGC10 connection: 0= not connected, 1= VG10 connected, 2= VGC10 connected
 
	.isConn_addr  = 0
	.isConn_ret  = 0

	;Set bit address
	.isConn_addr = 1

	;Set offset according primary (single) or secondary
	IF .isConn_inst == 1 THEN
		.isConn_addr = .isConn_addr + primR_bitOffset
	ELSE
		.isConn_addr = .isConn_addr + secR_bitOffset
	END


	;Get data
	.isConn_ret = BITS(.isConn_addr, 16)

	IF .isConn_ret <> 0 THEN

        IF .isConn_ret == 16 THEN
            .isConn_ret = 1  
        END

         IF .isConn_ret == 17 THEN
            .isConn_ret = 2
        END

    ELSE
        .isConn_ret = 0
    END

.END


.PROGRAM OR_VG10_getLim.pc( .getLim_inst, .getLim_ret ) 
;FUNCTION: Get actual current limit
;@param .getLim_inst gripper instance 1 = primary(or single), 2 = secondary
;@ret Current limit in mA

	.getLim_addr  = 0
	.getLim_ret  = 0

	;Set bit address
	.getLim_addr = 17

	;Set offset according primary (single) or secondary
	IF .getLim_inst == 1 THEN
		.getLim_addr = .getLim_addr + primR_bitOffset
	ELSE
		.getLim_addr = .getLim_addr + secR_bitOffset
	END
	
	;Get data
	.getLim_ret = BITS(.getLim_addr, 16)

.END


.PROGRAM OR_VG10_getVacA.pc( .getVacA_inst, .getVacA_ret) 
;FUNCTION: Get vacuum level of channel A
;@param .getVacA_inst gripper instance 1 = primary(or single), 2 = secondary
;@ret Vacuum level in %

	.getVacA_addr  = 0
	.getVacA_ret  = 0

	;Set bit address
	.getVacA_addr = 33

	;Set offset according primary (single) or secondary
	IF .getVacA_inst == 1 THEN
		.getVacA_addr = .getVacA_addr + primR_bitOffset
	ELSE
		.getVacA_addr = .getVacA_addr + secR_bitOffset
	END
	
	;Get data
	.getVacA_ret = BITS(.getVacA_addr, 16)

	.getVacA_ret = .getVacA_ret / 10 ;get % from 1/10% 
.END


.PROGRAM OR_VG10_getVacB.pc( .getVacB_inst, .getVacB_ret)
;FUNCTION: Get vacuum level of channel B
;@param .getVacB_inst gripper instance 1 = primary(or single), 2 = secondary
;@ret Vacuum level in %
 
	.getVacB_addr  = 0
	.getVacB_ret  = 0

	;Set bit address
	.getVacB_addr = 49

	;Set offset according primary (single) or secondary
	IF .getVacB_inst == 1 THEN
		.getVacB_addr = .getVacB_addr + primR_bitOffset
	ELSE
		.getVacB_addr = .getVacB_addr + secR_bitOffset
	END
	
	;Get data
	.getVacB_ret = BITS(.getVacB_addr, 16)

	.getVacB_ret = .getVacB_ret / 10 ;get % from 1/10%

.END

















.PROGRAM OR_RGx_move.pc(.move_inst , .move_width , .move_force , .move_wait)
;FUNCTION: Open/Close gripper to specific dictance (mm) with specific force (N)
;@param .move_inst gripper instance 1 = primary(or single), 2 = secondary
;@param .move_width open/close distance (mm)
;@param .move_force force (N)
;@param .move_wait shall we wait for command execution? 1=YES, 0=NO

	.move_addr  = 0
	.move_val  = 0
	.move_offsW  = 0
	.move_offsR  = 0

	;Set offset according primary (single) or secondary
	IF .move_inst == 1 THEN
		.move_offsW = primW_bitOffset
		.move_offsR =primR_bitOffset
	ELSE
		.move_offsW = secW_bitOffset
		.move_offsR = secR_bitOffset
	END

	;Set width
	.move_addr = 1 + .move_offsW 
	.move_val = .move_width*10 ;Width is sent as 1/10mm 

	BITS .move_addr, 16 = .move_val

	;Set force
	.move_addr = 17 + .move_offsW 
	.move_val = .move_force*10 ;Force is sent as 1/10N

	BITS .move_addr, 16 = .move_val

	;Start move
	BITS (33 + .move_offsW), 16 = 1  ;Reset motion stop bit, Set start motion bit
	TWAIT 0.1
	BITS (33 + .move_offsW), 16 = 0 ;Reset start motion bit

	;Wait for gripper motion if needed
	IF .move_wait == 1 THEN
		TWAIT 0.5
		SWAIT -(65 + .move_offsR)
	END

.END


.PROGRAM OR_RGx_isConn.pc(.isConn_inst, .isConn_ret) 
;FUNCTION: Check gripper connection
;@param .isConn_inst gripper instance 1 = primary(or single), 2 = secondary
;@ret Connection state: 0=not connected, 1=connected

	.isConn_addr  = 0
	.isConn_ret  = 0

	;Set bit address
	.isConn_addr = 1

	;Set offset according primary (single) or secondary
	IF .isConn_inst == 1 THEN
		.isConn_addr = .isConn_addr + primR_bitOffset
	ELSE
		.isConn_addr = .isConn_addr + secR_bitOffset
	END
	
	;Get data
	.isConn_ret = BITS(.isConn_addr, 16)
	
	IF .isConn_ret == 32 OR .isConn_ret == 33 THEN
		.isConn_ret = 1
	ELSE
		.isConn_ret = 0
	END

.END


.PROGRAM OR_RGx_isGrip.pc(.isGrip_inst, .isGrip_ret) 
;FUNCTION: Check grip detected
;@param .isGrip_inst gripper instance 1 = primary(or single), 2 = secondary
;@ret Grip detected: 0=no grip, 1=grip detected

	.isGrip_addr  = 0
	.isGrip_ret  = 0

	;Set bit address
	.isGrip_addr = 66

	;Set offset according primary (single) or secondary
	IF .isGrip_inst == 1 THEN
		.isGrip_addr = .isGrip_addr + primR_bitOffset
	ELSE
		.isGrip_addr = .isGrip_addr + secR_bitOffset
	END
	
	;Get bit status
	.isGrip_ret = -SIG(.isGrip_addr)
.END


.PROGRAM OR_RGx_isBusy.pc(.isBusy_inst, .isBusy_ret )
;FUNCTION: Check that is gripper busy or idle
;@param .isBusy_inst Gripper instance: 1=single or primary, 2: secondary in dual configuration
;@ret Gripper state: 0=idle, 1=busy

	.isBusy_addr  = 0
	.isBusy_ret  = 0

	;Set bit address
	.isBusy_addr = 65

	;Set offset according primary (single) or secondary
	IF .isBusy_inst == 1 THEN
		.isBusy_addr = .isBusy_addr + primR_bitOffset
	ELSE
		.isBusy_addr = .isBusy_addr + secR_bitOffset
	END
	
	;Get bit status
	.isBusy_ret = -SIG(.isBusy_addr)
.END


.PROGRAM OR_RGx_isSSOn.pc(.isSSOn_inst,  .isSSOn_ret) 
;FUNCTION: Check safety switch state
;@param .isSSOn_inst Gripper instance: 1=single or primary, 2: secondary in dual configuration
;@ret Safety switch state: 0=none of the safety switches are triggered, 1=safety switch triggered and the gripper is disabled

	.isSSOn_addr  = 0
	.isSSOn_offs  = 0
	.isSSOn_ret  = 0


	;Set offset according primary (single) or secondary
	IF .isSSOn_inst == 1 THEN
		.isSSOn_offs =  primR_bitOffset
	ELSE
		.isSSOn_offs =  secR_bitOffset
	END

	;Set bit address
	.isSSOn_addr = 68 + .isSSOn_offs  ;safety circuit 1
	;Get bit status
	.isSSOn_ret = -SIG(.isSSOn_addr)
	
	;If safety circuit 1 is not triggered, check the other one
	IF .isSSOn_ret == 0 THEN
		.isSSOn_addr = 70 + .isSSOn_offs  ;safety circuit 2
		
		;Get bit status
		.isSSOn_ret = -SIG(.isSSOn_addr)
	END
.END
